package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.vo.scoreRule.extra.ScoreRuleRegisterExtraVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class RegisterStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private MemberUserService memberUserService;

    public RegisterStrategy(ScoreRuleService scoreRuleService, MemberUserScoreLogService memberUserScoreLogService, MemberUserService memberUserService, MemberUserScoreApi memberUserScoreApi) {
        super(scoreRuleService, memberUserScoreLogService, memberUserService, memberUserScoreApi);
    }

    @Override
    public void addScore(ScoreMessage message) {
        Long userId = message.getUserId();
        log.info("Received register message,userID:{}", userId);
        ScoreRuleDO scoreRuleDO = scoreRuleService.getEnableScoreRuleByType(ScoreRuleTypeEnum.REGISTER);
        if (scoreRuleDO == null) {
            log.info("Register rule is not exist,userID:{}", userId);
            return;
        }
        MemberUserDO memberUserDO = memberUserService.getUser(userId);
        if (memberUserDO == null) {
            log.info("Register listener: The user does not exist,userID:{}", userId);
            return;
        }
        // 校验注册平台
        ScoreRuleRegisterExtraVO scoreRuleRegisterExtraVO = JsonUtils.parseObject(scoreRuleDO.getExtra(), ScoreRuleRegisterExtraVO.class);
        if (scoreRuleRegisterExtraVO != null && !"2,3".equals(scoreRuleRegisterExtraVO.getRegisterPlatform())) {
            if (!scoreRuleRegisterExtraVO.getRegisterPlatform().contains(String.valueOf(message.getRegisterPlatform()))) {
                log.info("Register listener: The register platform does not match,userID:{}", userId);
                return;
            }
        }
        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            map.put("userNameZh", message.getUserNameZh());
            map.put("userNameEn", message.getUserNameEn());
            String desensitizedPhoneNumber = message.getPhone().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
            map.put("phone", desensitizedPhoneNumber);

            memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(userId)
                    .scoreCount(scoreRuleDO.getGetScoreOnce())
                    .sourceType(ScoreSourceTypeEnum.REGISTER)
                    .ruleId(scoreRuleDO.getId())
                    .uniqueId(String.valueOf(userId))
                    .expireDays(scoreRuleDO.getScorePeriod())
                    .extParam(map)
                    .build());
        } catch (Exception e) {
            log.error("Register listener: operateScore error", e);
        }
        log.info("Register rule add score success,ruleID:{},userID:{}", scoreRuleDO.getId(), userId);
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.REGISTER;
    }
}
