package cn.iocoder.yudao.module.member.service.scoreLog;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.controller.app.userScore.vo.AppMemberUserIdQueryVO;
import cn.iocoder.yudao.module.member.controller.app.userScore.vo.AppMemberUserScoreLogRespVO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreLog.MemberUserScoreLogMapper;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogCreateReq;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogQueryVO;
import cn.iocoder.yudao.module.member.vo.scoreRule.ScoreRuleBackDetailVO;
import cn.iocoder.yudao.module.reward.api.reward.RedeemRewardApi;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemDetailForUserDTO;
import cn.iocoder.yudao.module.system.service.dict.DictTypeService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;

/**
 * 会员积分日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreLogServiceImpl extends AbstractService<MemberUserScoreLogMapper, MemberUserScoreLogDO> implements MemberUserScoreLogService {

    @Resource
    private MemberUserScoreLogMapper userScoreLogMapper;
    @Resource
    private SnowflakeGenerator snowflakeGenerator;
    @Resource
    private ScoreRuleService scoreRuleService;
    @Resource
    private RedeemRewardApi redeemRewardApi;


    @Override
    public PageResult<MemberUserScoreLogBackVO> getPage(MemberUserScoreLogQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreLogBackVO> list = userScoreLogMapper.getPageRecordList(start, size, query);
        int total = userScoreLogMapper.getPageCount(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Long createScoreLog(MemberUserScoreLogCreateReq createReq) {
        if (createReq.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreLogDO userScoreLogDO = new MemberUserScoreLogDO();
        userScoreLogDO.setId(snowflakeGenerator.next());
        userScoreLogDO.setMemberId(createReq.getMemberId());
        userScoreLogDO.setScoreCount(createReq.getScoreCount());
        userScoreLogDO.setOperateType(createReq.getOperateType());
        userScoreLogDO.setSourceType(createReq.getSourceType());
        userScoreLogDO.setRuleId(createReq.getRuleId());
        userScoreLogDO.setUniqueId(createReq.getUniqueId());
        if (createReq.getExtParam() != null) {
            userScoreLogDO.setExtParam(JSONUtil.toJsonStr(createReq.getExtParam()));
        }
        this.saveOrUpdate(userScoreLogDO);
        return userScoreLogDO.getId();
    }

    @Override
    public MemberUserScoreLogDO getByUniqueId(String uniqueId) {
        LambdaQueryWrapper<MemberUserScoreLogDO> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MemberUserScoreLogDO::getUniqueId, uniqueId);
        return this.selectOne(wrapper);
    }

    @Override
    public List<AppMemberUserScoreLogRespVO> getUserScoreLogList(AppMemberUserIdQueryVO query) {
        List<MemberUserScoreLogDO> userScoreLogDOS = userScoreLogMapper.selectList("member_id", query.getId());
        if (userScoreLogDOS == null || userScoreLogDOS.isEmpty()) {
            return Collections.emptyList();
        }
        Map<Long, MemberUserScoreLogDO> scoreLogDOMap = userScoreLogDOS.stream()
                .collect(Collectors.toMap(MemberUserScoreLogDO::getId, log -> log));

        List<AppMemberUserScoreLogRespVO> appMemberUserScoreLogRespVOS = new ArrayList<>();

        for (MemberUserScoreLogDO userScoreLogDO : userScoreLogDOS) {
            if (userScoreLogDO.getSourceType() == ScoreSourceTypeEnum.SYSTEM_EXPIRED.getValue()) {
                Map extParams = JsonUtils.parseObject(userScoreLogDO.getExtParam(), Map.class);
                List<Long> scoreLogIdList = JsonUtils.parseArray(extParams.get("scoreLogIds").toString(), Long.class);

                scoreLogIdList.stream()
                        .map(scoreLogDOMap::get)
                        .filter(scoreLogDO -> scoreLogDO != null && scoreLogDO.getRuleId() != null)
                        .forEach(scoreLogDO -> {
                            ScoreRuleBackDetailVO scoreRule = scoreRuleService.getScoreRule(scoreLogDO.getRuleId());
                            if (scoreRule != null) {
                                appMemberUserScoreLogRespVOS.add(createScoreLogRespVO(userScoreLogDO, scoreLogDO, scoreRule));
                            }
                        });
            } else {
                setScoreRuleLogDetail(userScoreLogDO, appMemberUserScoreLogRespVOS);
            }
        }
        // 根据创建时间倒序
        appMemberUserScoreLogRespVOS.sort(Comparator.comparing(AppMemberUserScoreLogRespVO::getCreateTime).reversed());
        return appMemberUserScoreLogRespVOS;
    }

    private AppMemberUserScoreLogRespVO createScoreLogRespVO(MemberUserScoreLogDO userScoreLogDO,
                                                             MemberUserScoreLogDO scoreLogDO,
                                                             ScoreRuleBackDetailVO scoreRule) {
        return new AppMemberUserScoreLogRespVO(
                userScoreLogDO.getId(),
                userScoreLogDO.getScoreCount(),
                "-",
                scoreLogDO.getSourceType(),
                DictFrameworkUtils.getDictDataFromCache("member_score_source", scoreLogDO.getSourceType().toString()).getLabel(),
                DictFrameworkUtils.getDictDataFromCache("member_score_source", scoreLogDO.getSourceType().toString()).getLabelEn(),
                userScoreLogDO.getCreateTime(),
                scoreRule.getDescEn(),
                scoreRule.getDescZh(),
                null,
                true
        );
    }


    private void setScoreRuleLogDetail(MemberUserScoreLogDO userScoreLogDO, List<AppMemberUserScoreLogRespVO> appMemberUserScoreLogRespVOS) {
        AppMemberUserScoreLogRespVO appMemberUserScoreLogRespVO = new AppMemberUserScoreLogRespVO();
        appMemberUserScoreLogRespVO.setId(userScoreLogDO.getId());
        appMemberUserScoreLogRespVO.setSourceType(userScoreLogDO.getSourceType());
        appMemberUserScoreLogRespVO.setScoreCount(userScoreLogDO.getScoreCount());
        appMemberUserScoreLogRespVO.setOperateType(userScoreLogDO.getOperateType() == 1 ? "+" : "-");
        appMemberUserScoreLogRespVO.setCreateTime(userScoreLogDO.getCreateTime());

        // 获取通用的字典标签
        String sourceType = userScoreLogDO.getSourceType().toString();
        String sourceTypeZh = DictFrameworkUtils.getDictDataFromCache("member_score_source", sourceType).getLabel();
        String sourceTypeEn = DictFrameworkUtils.getDictDataFromCache("member_score_source", sourceType).getLabelEn();
        appMemberUserScoreLogRespVO.setSourceTypeZh(sourceTypeZh);
        appMemberUserScoreLogRespVO.setSourceTypeEn(sourceTypeEn);

        // 判断是增加还是减少积分操作
        boolean isAddOperation = Objects.equals(userScoreLogDO.getOperateType(), ScoreOperateTypeEnum.ADD.getValue());

        if (isAddOperation) {
            handleAddScoreLogDetail(userScoreLogDO, appMemberUserScoreLogRespVO);
        } else {
            handleReduceScoreLogDetail(userScoreLogDO, appMemberUserScoreLogRespVO);
        }

        appMemberUserScoreLogRespVOS.add(appMemberUserScoreLogRespVO);
    }

    private void handleAddScoreLogDetail(MemberUserScoreLogDO userScoreLogDO, AppMemberUserScoreLogRespVO appMemberUserScoreLogRespVO) {
        switch (userScoreLogDO.getSourceType()) {
            case 1:
                appMemberUserScoreLogRespVO.setDescZh("人工积分");
                appMemberUserScoreLogRespVO.setDescEn("manually add points");
                break;
            case 4:
                handleRedeemDetail(userScoreLogDO, appMemberUserScoreLogRespVO);
                break;
            case 5:
            case 6:
            case 7:
            case 8:
                ScoreRuleBackDetailVO scoreRule = scoreRuleService.getScoreRule(userScoreLogDO.getRuleId());
                if (scoreRule != null) {
                    appMemberUserScoreLogRespVO.setDescEn(scoreRule.getDescEn());
                    appMemberUserScoreLogRespVO.setDescZh(scoreRule.getDescZh());
                }
                break;
        }
    }

    private void handleReduceScoreLogDetail(MemberUserScoreLogDO userScoreLogDO, AppMemberUserScoreLogRespVO appMemberUserScoreLogRespVO) {
        switch (userScoreLogDO.getSourceType()) {
            case 1:
                appMemberUserScoreLogRespVO.setDescZh("人工积分");
                appMemberUserScoreLogRespVO.setDescEn("manually reduce points");
                break;
            case 2:
                handleRedeemDetail(userScoreLogDO, appMemberUserScoreLogRespVO);
                break;
        }
    }

    private void handleRedeemDetail(MemberUserScoreLogDO userScoreLogDO, AppMemberUserScoreLogRespVO appMemberUserScoreLogRespVO) {
        HashMap extParam = JsonUtils.parseObject(userScoreLogDO.getExtParam(), HashMap.class);
        Long redeemId = extParam.get("redeemId") != null ? Long.valueOf(extParam.get("redeemId").toString()) : null;

        if (redeemId != null) {
            RedeemDetailForUserDTO redeemDetailForUser = redeemRewardApi.getRedeemDetailForUser(redeemId);
            if (redeemDetailForUser != null) {
                appMemberUserScoreLogRespVO.setRewardCount(redeemDetailForUser.getRewardCount());
                appMemberUserScoreLogRespVO.setDescZh(redeemDetailForUser.getRewardNameZh());
                appMemberUserScoreLogRespVO.setDescEn(redeemDetailForUser.getRewardNameEn());
            }
        }
    }
}
